/*
Research Project:   Graphical Database for Category Theory
                    J. Bradbury, Dr. R. Rosebrugh, I. Rutherford
                    M. Graves, J.Tweedle
                    Mount Allison University: July 2003
File:               AddDataFrame.java
Description:	      An internal frame that gives the user the ability
                    to add data to the category.
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;


public class AddDataFrame extends JInternalFrame
{
	private CategoryInternalFrame categoryInternalFrame_; // stores the parent internal frame
	private Category category_;		// stores the category to be modified
	private GMLEditor gmlEdit_;		// used to edit the gml
	private JButton doneButton;		// done button: applies the changes made
	private JButton cancelButton;	// cancel button: cancels all changes made
	private JTabbedPane tabs;		// tabs used to display elements (objects, arrows, etc.)
	private JLabel categoryLabel;	// displays category name label
	private JTextField categoryField;// displays (and allows changes to) category name
	private Container contentPane;
	private InputCheck ic;			// checks the user's input for special characters

	private int num_objects;		// the new number of objects (original + number added)
	private String[] objects;		// the names of the objects to be checked and added to the category
	private JPanel objectPanel;		// the "objects" tab
	private JButton addObjectButton;// add button: checks the name and adds the object to the category
	private JTextArea objectArea;	// displays the objects in the category (including new and old ones)
	private JLabel objectLabel;		// displays the "Object Name:" label
	private JTextField objectField;	// used to input the name of the object
	private JScrollPane objectPane;	// wraps "objectArea"; need this to use scrollbars

	private int num_arrows;			// the new number of arrows (original + number added)
	private int[][] arrDC;			// the co/domains of the arrows
	private String[] arrows;		// the names of the arrows to be checked and added to the category
	private JPanel arrowPanel;		// the "arrows" tab
	private JButton addArrowButton;	// add button: checks the name and adds the object to the category
	private JTable arrowTable;		// displays the (names, co/do) arrows in the category (including new and old)
	private JComboBox domainBox;	// drop-down list to pick the domain of the arrow to be added
	private JComboBox codomainBox;	// drop-down list to pick the codomain of the arrow to be added
	private JLabel arrowLabel;		// displays the "Arrow Name:" label
	private JLabel domainLabel;		// displays the "Domain:" label
	private JLabel codomainLabel;	// displays the "Codomain:" label
	private JTextField arrowField;	// used to input the name of the arrow
	private JScrollPane arrowPane;	// wraps "arrowTable"; need this to use scrollbars

	private int num_relations;		// the new number of relations (original + number added)
	private String[] relations;		// the relations to be checked and added to the category
	private JPanel relationPanel;	// the "relations" tab
	private JButton addRelationButton;// add button: checks the relation and adds it to the category
	private JLabel relationLabel;	// displays the "Relation:" label
	private JTextField relationField;// used to input the relation
	private JTextArea relationArea;	// displays the relations in the category (including new and old)
	private JScrollPane relationPane;// wraps "relationArea"; need this to use scrollbars

	private JPanel commentPanel;	// the "comments" tab
	private JLabel commentLabel;	// displays the "Comments:" label
	private JTextArea commentArea;	// displays the comments (including new and old)
	private JScrollPane commentPane;// wraps "commentArea"; need this to use scrollbars

	public AddDataFrame(CategoryInternalFrame categoryInternalFrame)
	/* 	AddDataFrame constructor:
				- takes a CategoryInternalFrame
				- constructs an JInternalFrame, adds it to the CIF's desktop
				- displays all current category information in appropriate tab
				- displays all labels, buttons, text fields for user
				- allows user to add objects, arrows, relations and comments to
					the category
				- allows user to change the name of the category
				- allows user to discard changes by clicking "Cancel"
				- allows user to apply changes by clicking "Done"
				- uses InputCheck.java (for name checking purposes) 
					and GMLEditor.java (for adding the specified elements to the GML) 
	*/
	{
		super("Add Data to Category",	// call to the constructor of JInternalFrame
				true, //resizable
				true, //closable
				false, //maximizable
				false);//iconifiable

		categoryInternalFrame_ = categoryInternalFrame;	// store the parent internal frame
		category_ = categoryInternalFrame_.getCategory(); // store the category to be modified
		num_objects = category_.num_objects;	// set num_objects to be the original number of objects
		objects = category_.obj;	// set objects to be the original objects
		num_arrows = category_.num_arrows;	// set num_arrows to be the original number of arrows
		arrows = category_.arrow;	// set arrows to be the original arrows
		arrDC = category_.arr;	// set arrDC to be the original co/domains
		num_relations = category_.num_relations; // set num_relations to be the original number of relations
		relations = new String[category_.ini.getRELLEN()];	// relations that will be added
		gmlEdit_ = new GMLEditor();	// used to add the new elements to the category's GML
		ic = new InputCheck(); // used to check the names of the new elements


		/* construct tabs and category name/label/textfield/buttons */
		contentPane = getContentPane();
		tabs = new JTabbedPane(JTabbedPane.TOP);		
		categoryLabel = new JLabel("Category Name:");
		categoryField = new JTextField(category_.name);
		doneButton = new JButton("Done");
		doneButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				doneButton_actionPerformed(e);
			}
		});
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
			{
				cancelButton_actionPerformed(e);
			}
		});
		/* end constructing tabs and category information */



		/* construct all GUI objects in "arrow" tab */
		arrowLabel = new JLabel("Arrow(s):");
		domainLabel = new JLabel("Domain:");
		codomainLabel = new JLabel("Codomain:");
		arrowField = new JTextField(15);
		addArrowButton = new JButton("Add Arrow");
		addArrowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				addArrowButton_actionPerformed(e);
			}
		});

		domainBox = new JComboBox();
		codomainBox = new JComboBox();
		domainBox.setEditable(false);
		codomainBox.setEditable(false);
		domainBox.setBackground(Color.white);
		codomainBox.setBackground(Color.white);	

		String[][] data = new String[50][3];
		for (int i=0; i<50; i++)
		{
			for (int j=0; j<3; j++)
			{
				if (i < num_arrows) {
					data[i][0] = arrows[i];
					data[i][1] = objects[arrDC[i][0]];
					data[i][2] = objects[arrDC[i][1]];
				}
				else
					data[i][0] = data[i][1] = data[i][2] = "";
			}
		}

		String[] columnNames = {"Arrow Name", 
								"Domain of Arrow",
								"Codomain of Arrow"};
		arrowTable = new JTable(data, columnNames);
		arrowPane = new JScrollPane(arrowTable);
		arrowPane.setBorder(BorderFactory.createEtchedBorder());
		/* end constructing GUI objects for "arrows" tab */



		/* construct GUI objects for "objects" tab */
		objectLabel = new JLabel("Object(s):");
		objectField = new JTextField(15);
		addObjectButton = new JButton("Add Object");
		addObjectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				addObjectButton_actionPerformed(e);
			}
		});

		objectArea = new JTextArea();
		objectArea.setWrapStyleWord(true);
		objectArea.setEditable(false);
		for (int i=0; i<num_objects; i++)
		{
			domainBox.addItem(objects[i]);
			codomainBox.addItem(objects[i]);			
			if (i<num_objects-1)
				objectArea.append(objects[i] + ", ");
			else
				objectArea.append(objects[i] + "");
		}
		objectPane = new JScrollPane(objectArea);
		objectPane.setBorder(BorderFactory.createEtchedBorder());
		/* end constructing GUI objects for "objects" tab */
	
	

		/* construct GUI objects for "relations" tab */
		relationLabel = new JLabel("Relation(s):");
		relationField = new JTextField(15);
		addRelationButton = new JButton("Add Relation");
		addRelationButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				addRelationButton_actionPerformed(e);
			}
		});

		relationArea = new JTextArea(category_.relationsToString());
		relationPane = new JScrollPane(relationArea);
		relationPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		relationArea.setLineWrap(true);
		relationArea.setWrapStyleWord(true);
		relationArea.setEditable(false);
		relationPane.setBorder(BorderFactory.createEtchedBorder());
		/* end constructing GUI objects for "relations" tab */
	

		/* construct GUI objects for "comments" tab */	
		commentLabel = new JLabel("Comment(s):");							
		commentArea = new JTextArea(category_.comment);
		commentPane = new JScrollPane(commentArea);
		commentPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		commentArea.setLineWrap(true);
		commentArea.setWrapStyleWord(true);
		commentPane.setBorder(BorderFactory.createEtchedBorder());
		/* end constructing GUI objects for "comments" tab */



		/* add all GUI objects for "objects" tab to the "objects" tab */
		objectPanel = new JPanel(new GridBagLayout());					
		objectPanel.add(objectLabel, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.1
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
		objectPanel.add(objectField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.1
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		objectPanel.add(addObjectButton, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.1
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
		objectPanel.add(objectPane, new GridBagConstraints(0, 1, 10, 10, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));



		/* add all GUI objects for "arrows" tab to the "arrows" tab */
		arrowPanel = new JPanel(new GridBagLayout());	

		arrowPanel.add(arrowLabel,  new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
		arrowPanel.add(domainLabel, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
		arrowPanel.add(codomainLabel, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));

		arrowPanel.add(arrowField, new GridBagConstraints(0, 1, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		arrowPanel.add(domainBox, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		arrowPanel.add(codomainBox, new GridBagConstraints(2, 1, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));

		arrowPanel.add(addArrowButton, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));

		arrowPanel.add(arrowPane,  new GridBagConstraints(0, 3, 10, 10, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));




		/* add all GUI objects for "relations" tab to the "relations" tab */
		relationPanel = new JPanel(new GridBagLayout());	

		relationPanel.add(relationLabel, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
		relationPanel.add(relationField, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		relationPanel.add(addRelationButton, new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(3, 3, 3, 3), 0, 0));
		relationPanel.add(relationPane, new GridBagConstraints(0, 1, 3, 3, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));



		/* add all GUI objects for "comments" tab to the "comments" tab */
		commentPanel = new JPanel(new BorderLayout());				
		commentPanel.add(commentLabel, BorderLayout.NORTH);
		commentPanel.add(commentPane, BorderLayout.CENTER);


		/* add all tabs to the actual JTabbedPane */
		tabs.add(objectPanel, "Objects");
		tabs.add(arrowPanel, "Arrows");
		tabs.add(relationPanel, "Relations");
		tabs.add(commentPanel, "Comments");
	

		/* add al GUI objects to the actual JInternalFrame */
		contentPane.setLayout(new GridBagLayout());
// -- Tabs -- //
		contentPane.add(tabs, new GridBagConstraints(0, 0, 10, 1, 1.0, 1.0
		,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(3, 3, 3, 3), 0, 0));

// -- Category Name/Field/Button Info -- //
		contentPane.add(categoryLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
		,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 3, 3, 3), 0, 0));
		contentPane.add(categoryField, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0
		,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(7, 7, 7, 7), 0, 0));				
		contentPane.add(doneButton, new GridBagConstraints(2, 1, 1, 1, 0.5, 0.0
		,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(7, 7, 7, 7), 0, 0));
		contentPane.add(cancelButton, new GridBagConstraints(3, 1, 1, 1, 0.5, 0.0
		,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(7, 7, 7, 7), 0, 0));	
	} // ------- End of CreateCategoryDialog constructor ------------------------


		
		
		
	private void addObjectButton_actionPerformed(ActionEvent e)
	/* 	This method called when "addObjectButton" is clicked.
			- checks the name of the object
			- adds it to the list of objects (including new and old)
			- increments num_objects
			- adds it to the object text area
			- adds it to the lists (co/domains) of objects in the "arrow" tab
			- resets the textfield
		*** note: this method does not actually add the object to the category ***
	*/
	{
		String objectName = objectField.getText();

		if (ic.check(objectName))	// check the object name
		{
			objects[num_objects++] = objectName;	// add to the list of objects and increment num_objects
	
			objectArea.append(", " + objectName);	// add to the textarea
	
			// add object to do/codomain combo boxes
			domainBox.addItem(objectName);
			codomainBox.addItem(objectName);
	
			objectField.setText("");	// reset the textfield
		}
	}





	private void addArrowButton_actionPerformed(ActionEvent e)
	/* 	This method called when "addArrowButton" is clicked.
			- checks the name of the arrow
			- adds it to the list of arrows (including new and old)
			- adds it to the arrow table
			- increments num_arrows
			- resets the textfield
		*** note: this method does not actually add the arrow to the category ***
	*/
	{
		String arrowName = arrowField.getText();
		int domainIndex = domainBox.getSelectedIndex();
		int codomainIndex = codomainBox.getSelectedIndex();
		// check arrow name/do/codomain for other stuff
		
		if (ic.check(arrowName))
		{
			arrows[num_arrows] = arrowName;
			arrDC[num_arrows][0] = domainIndex;
			arrDC[num_arrows][1] = codomainIndex;
	
			arrowTable.setValueAt(arrowName, num_arrows, 0);
			arrowTable.setValueAt((String)domainBox.getSelectedItem(), num_arrows, 1);
			arrowTable.setValueAt((String)codomainBox.getSelectedItem(), num_arrows++, 2); 
	
			arrowField.setText("");
		}
	}
	
	
	
	private void addRelationButton_actionPerformed(ActionEvent e)
	/* 	This method called when "addRelationButton" is clicked.
			- checks the name of the relation
			- adds it to the list of relations
			- adds it to the relation text area
			- increments num_relations
			- resets the textfield
		*** note: this method does not actually convert the string to a relation,
				check it for validity, or  add the relation to the category ***
	*/
	{
		String relationName = relationField.getText();
		
		relations[num_relations++] = relationName;	// add relation to list of new relations
		
		relationArea.append(relationName + "\n");	// add relation to text area
		relationField.setText("");	// reset text field
	}

	private void cancelButton_actionPerformed(ActionEvent e)
	/* 	This method called when "cancelButton" is clicked.
			- cancels all modifications; does not add anything to the category
			- closes AddDataFrame window
	*/	
	{
		try {this.setClosed(true);}	// close the window
		catch (PropertyVetoException PVE) {}
	}

	private void doneButton_actionPerformed(ActionEvent e)
	/* 	This method called when "doneButton" is clicked.
			- checks the name of the category;
			- call setObjects(), setArrows(), setRelations()
				and setComments(); this adds the information
				to the category
			- closes AddDataFrame window
	*/	
	{
		String name = categoryField.getText();
		// add all other stuff to new category with cat.name = name

		if (ic.check(name))	// check the name for special characters
		{
			category_.name = name;	// set the name of the category
		
			setObjects();	// set the objects
	
			setArrows();	// set the arrows

			setRelations();	// set the relations
	
			setComments();	// set the comments
	
			category_.mod.dataAdded = true;	// make sure the category knows it has been modified;
											// need this for saving purposes
			categoryInternalFrame_.setCategory(category_);	// set the category
			try {this.setClosed(true);}	// close the window
			catch (PropertyVetoException PVE) {}
		}
	}

	private void setObjects()
	/* This method:
			- adds all objects to the category and the category's gml
	*/
	{
		int temp = category_.num_objects;
		for (int i=temp; i<num_objects; i++)
		{
			category_.obj[category_.num_objects] = objects[category_.num_objects++];
			gmlEdit_.addObject(category_);
		}
	}

	private void setArrows()
	/* This method:
			- adds all arrows to the category and the category's gml
	*/
	{
		int temp = category_.num_arrows;
		for (int i=temp; i<num_arrows; i++)
		{
			category_.arrow[category_.num_arrows] = arrows[category_.num_arrows];
			category_.arr[category_.num_arrows][0] = arrDC[category_.num_arrows][0];
			category_.arr[category_.num_arrows][1] = arrDC[category_.num_arrows++][1];
			gmlEdit_.addArrow(category_);
		}
	}

	private void setRelations()
	/* This method:
			- adds all relations to the category and the category's gml
	*/
	{
		for (int i=category_.num_relations; i<num_relations; i++)
			gmlEdit_.addRelation(category_, relations[i]);
			
		Relation[] relations_ = new Relation[category_.num_relations + num_relations]; //category_.relations;
		for (int i=0; i<category_.num_relations; i++)
			relations_[i] = category_.relations[i];
		for (int i=category_.num_relations; i<num_relations; i++)
			relations_[i] = category_.get_rel(relations[i]);
			
		category_.relations = relations_;
		category_.num_relations = num_relations;
	}
			
	private void setComments()
	/* This method:
			- adds all relations to the category
	*/
	{
		category_.comment = commentArea.getText();
	}
}